import sys
from pyspark.context import SparkContext
from awsglue.context import GlueContext
from awsglue.utils import getResolvedOptions
from awsglue.job import Job


def main() -> None:

    expected_args = ["JOB_NAME"]
    args = getResolvedOptions(sys.argv, expected_args)

    spark_context = SparkContext.getOrCreate()
    glue = GlueContext(spark_context)
    spark = glue.spark_session

    # Init the glue job
    glue_job = Job(glue)
    glue_job.init(args["JOB_NAME"], args)

    # mode code here....


if __name__ == "__main__":
    main()



from typing import Optional
from superglue.core.components.base import SuperglueComponentType
from superglue.core.components.job import SuperglueJobType
from superglue.core.components.module import SuperglueModuleType


class Messages:
    def __init__(self, component: SuperglueComponentType) -> None:
        self.component = component

    @staticmethod
    def project_initialized() -> None:
        print("Superglue project has been created successfully!")

    @staticmethod
    def modules_locked() -> None:
        print("All superglue modules are locked")

    @staticmethod
    def modules_not_locked() -> None:
        print("Not all superglue modules are locked. Please lock them with the superglue lock command.")

    @staticmethod
    def jobs_locked() -> None:
        print("All superglue jobs are locked.")

    @staticmethod
    def jobs_not_locked() -> None:
        print("Not all superglue jobs are locked. Please lock them with the superglue lock command.")

    @staticmethod
    def packaging_module(name: str, dry: Optional[bool] = False) -> None:
        pre = ""
        if dry:
            pre = "-- Dry Run -- "
        print(f"{pre}Packaging superglue module :: {name}")

    @staticmethod
    def locking_jobs() -> None:
        print("\n------------------> Locking Superglue Jobs <------------------\n")

    @staticmethod
    def locking_modules() -> None:
        print("\n------------------> Locking Superglue Modules <------------------\n")

    @staticmethod
    def locking_job(job: SuperglueJobType) -> None:
        print(f"Locking Job -----> {job.name} :: Version {job.version_number}")

    @staticmethod
    def locking_module(module: SuperglueModuleType) -> None:
        print(f"Locking module -----> {module.name} :: Version {module.version_number}")

    @staticmethod
    def no_deployment() -> None:
        print("Unlocked modules or jobs exist. Deployment is not possible.")

    @staticmethod
    def updating_local_version(component: SuperglueComponentType, version: int):
        print(f"Updating local version for {component.component_type} {component.name} to version {version}")

    @staticmethod
    def versions_in_sync(component: SuperglueComponentType, version: int):
        print(
            f"Remote version {version} for {component.component_type} {component.name} is in sync with local version."
        )

    @staticmethod
    def version_missmatch() -> None:
        print("Versions between local and S3 don't match. Run superglue refresh to fix this problem.")

    @staticmethod
    def yes_deployment() -> None:
        print("All superglue jobs and modules are up to date. Deployment is possible.")

    @staticmethod
    def removed_zipfile(module: SuperglueModuleType) -> None:
        print(f"Removed zip package for superglue module {module.name}")

    @staticmethod
    def purge_complete() -> None:
        print("Purging superglue module zipfiles done.")

    @staticmethod
    def packaging_complete() -> None:
        print("Packaging superglue modules done.")

    @staticmethod
    def nothing_to_deploy() -> None:
        print("Everything up to date in S3. Nothing to Deploy.")

    @staticmethod
    def dry_run() -> None:
        print("\n----------------------->> DEPLOYMENT DRY RUN <<-----------------------\n")

    @staticmethod
    def job_deploy(job: SuperglueJobType, dry: Optional[bool] = False) -> None:
        pre = ""
        if dry:
            pre = "-- Dry Run -- "

        print(f"{pre}Superglue job {job.name} deployed to the following location.")
        print(job.s3_path)
        print("\n")

    @staticmethod
    def module_deploy(module: SuperglueModuleType, dry: Optional[bool] = False) -> None:
        pre = ""
        if dry:
            pre = "-- Dry Run -- "

        print(f"{pre}Superglue module {module.name} deployed to the following location.")
        print(module.s3_path)
        print("\n")

    @staticmethod
    def not_packaged() -> None:
        print("There are unpackaged modules present. Deployment not possible.")

    @staticmethod
    def all_jobs_locked() -> None:
        print("All superglue jobs are locked")

    @staticmethod
    def all_modules_locked() -> None:
        print("All superglue modules are locked")

    def component_exists(self) -> None:
        print(f"{self.component.component_type.capitalize()} {self.component.component_name} already exists.")

    def component_created(self) -> None:
        print(
            f"{self.component.component_type.capitalize()} {self.component.component_name} has been created successfully."
        )



import zipfile
from pathlib import Path
from typing import Optional, TypeVar, List
from superglue.environment.config import MODULES_PATH
from superglue.core.components.base import SuperglueComponent
from superglue.core.components.tests import SuperglueTests
from superglue.environment.variables import SUPERGLUE_S3_BUCKET, SUPERGLUE_IAM_ROLE


SuperglueModuleType = TypeVar("SuperglueModuleType", bound="SuperglueModule")


class SuperglueModule(SuperglueComponent):
    def __init__(
        self,
        name: str,
        version_number: Optional[int] = None,
        tests: Optional[SuperglueTests] = None,
        *args,
        **kwargs,
    ) -> None:

        self.tests = tests or SuperglueTests()

        super(SuperglueModule, self).__init__(
            *args,
            root_dir=MODULES_PATH,
            component_name=name,
            component_type="superglue_module",
            bucket=SUPERGLUE_S3_BUCKET,
            iam_role=SUPERGLUE_IAM_ROLE,
            **kwargs,
        )

        if version_number:
            self.version_number = version_number

    @property
    def module_root_path(self) -> Path:
        return self.root_dir / self.name

    @property
    def module_inner_path(self) -> Path:
        return self.module_root_path / self.name

    @property
    def zipfile(self) -> Path:
        return self.module_root_path / f"{self.name}.zip"

    @property
    def s3_zipfile_path(self) -> str:
        relative_path = self.zipfile.relative_to(self.module_root_path)
        return f"{self.s3_path}/{relative_path}"

    @property
    def module_test_path(self) -> Path:
        return self.tests.modules_test_dir / self.name

    @property
    def module_tests_file(self):
        return self.module_test_path / f"test_{self.name}.py"

    @property
    def is_packaged(self) -> bool:
        return self.zipfile.exists()

    @classmethod
    def new(cls, module_name: str) -> SuperglueModuleType:
        return cls(module_name)

    @classmethod
    def get(cls, module_name: str) -> SuperglueModuleType:
        sg_module = cls(module_name)
        if not sg_module.module_root_path.exists():
            raise FileNotFoundError(f"No superglue module {module_name} exists.")
        return sg_module

    @classmethod
    def from_version(cls, module_name: str, version_number: int) -> SuperglueModuleType:
        sg_module = cls(name=module_name, version_number=version_number)
        if not sg_module.module_root_path.exists():
            raise FileNotFoundError(f"No superglue module {module_name} exists.")
        return sg_module

    def module_files(self) -> List[Path]:
        return list(self.module_root_path.glob("**/*.py"))

    def save(self) -> None:
        self.module_inner_path.mkdir(parents=True, exist_ok=True)
        init_py = self.module_inner_path / "__init__.py"
        init_py.touch(exist_ok=True)
        self.save_version_file()
        self.save_tests()

    def deploy(self, increment_version: bool) -> None:
        if increment_version:
            self.append_version()
        self.sync()

    def delete(self) -> None:
        raise NotImplementedError

    def zipfile_relative_path(self, path: Path) -> str:
        return path.relative_to(self.module_root_path).as_posix()

    def zipfile_content(self, path: Path) -> str:
        return path.read_text(encoding="utf-8")

    def package(self) -> None:
        with zipfile.ZipFile(self.zipfile, mode="w") as zip_file:
            for file in self.module_files():
                content = self.zipfile_content(file)
                relative_path = self.zipfile_relative_path(file)
                zip_file.writestr(relative_path, content)

    def remove_zipfile(self) -> None:
        if self.zipfile.exists():
            self.zipfile.unlink()

    def save_tests(self) -> None:
        jinja = self.get_jinja_environment()
        tests_template = jinja.get_template("module_test.template.py.txt")
        test_content = tests_template.render(module=self.name)

        self.module_test_path.mkdir(exist_ok=True, parents=True)
        self.module_tests_file.touch(exist_ok=True)
        self.module_tests_file.write_text(test_content)



from pathlib import Path

SUPERGLUE_CWD = Path.cwd()

JOBS_PATH = SUPERGLUE_CWD / "jobs"
MODULES_PATH = SUPERGLUE_CWD / "modules"
NOTEBOOKS_PATH = SUPERGLUE_CWD / "notebooks"
TOOLS_PATH = SUPERGLUE_CWD / "tools"
TESTS_PATH = SUPERGLUE_CWD / "tests"



import re
import boto3
import json
import botocore
from io import BytesIO
from pathlib import Path
from hashlib import md5
from abc import ABC, abstractmethod
from multiprocessing import Pool, cpu_count
from jinja2 import Environment, PackageLoader
from typing import Optional, Dict, Tuple, List, TypeVar, Generator
from superglue.environment.variables import SUPERGLUE_IAM_ROLE, SUPERGLUE_S3_BUCKET, SUPERGLUE_S3_PREFIX


BaseSuperglueComponentType = TypeVar(name="BaseSuperglueComponentType", bound="BaseSuperglueComponent")


class BaseSuperglueComponent(ABC):
    def __init__(self, root_dir: Path, component_name: str, component_type: str) -> None:

        self.root_dir = root_dir
        self.component_name = component_name
        self.component_type = component_type

    def __eq__(self, other: BaseSuperglueComponentType) -> bool:
        try:
            return self.component_name == other.component_name
        except AttributeError:
            return False

    @property
    def component_path(self) -> Path:
        return self.root_dir / self.component_name

    @staticmethod
    def get_jinja_environment() -> Environment:
        """
        used to get a jinja2 templating environment.
        Returns: Jinja2 Environment
        """
        loader = PackageLoader(package_name="superglue", package_path="templates")
        return Environment(loader=loader, trim_blocks=True, lstrip_blocks=True)

    @abstractmethod
    def save(self) -> None:
        pass


SuperglueComponentType = TypeVar("SuperglueComponentType", bound="SuperglueComponent")


class SuperglueComponent(BaseSuperglueComponent, ABC):

    version_pattern = r".*(version=[0-9]+).*"

    def __init__(
        self, bucket: Optional[str] = SUPERGLUE_S3_BUCKET, iam_role: Optional[str] = SUPERGLUE_IAM_ROLE, *args, **kwargs
    ) -> None:
        super(SuperglueComponent, self).__init__(*args, **kwargs)

        self.bucket = bucket
        self.iam_role = iam_role

        try:
            self.version = json.load(self.version_file.open())
        except FileNotFoundError:
            self.version = {}

        try:
            current_version = self.version.pop("version_number")
        except KeyError:
            current_version = 0

        self.version_number = current_version

    @property
    def name(self) -> str:
        return self.component_name

    @property
    def version_file(self) -> Path:
        return self.component_path / ".version"

    @property
    def s3_prefix_root(self) -> str:
        if SUPERGLUE_S3_PREFIX:
            return f"{SUPERGLUE_S3_PREFIX}/superglue"
        return f"superglue"

    @property
    def s3_path(self) -> str:
        return f"s3://{self.bucket}/{self.s3_prefix_root}/{self.component_type}/{self.component_name}/version={self.version_number}/{self.component_name}"

    @property
    def s3_prefix(self) -> str:
        return f"{self.s3_prefix_root}/{self.component_type}/{self.component_name}/version={self.version_number}"

    @property
    def s3_filter(self) -> str:
        return f"{self.s3_prefix_root}/{self.component_type}/{self.component_name}"

    @property
    def s3_version_path(self) -> str:
        return f"{self.s3_prefix}/{self.component_name}/.version"

    @property
    def is_locked(self) -> bool:
        return self.version == self.get_version_hashes()

    @property
    def is_unlocked(self) -> bool:
        return not self.is_locked

    @property
    def is_deployable(self) -> bool:
        s3_version = self.fetch_s3_version()
        local_version = self.version.copy()
        local_version["version_number"] = self.version_number
        return self.is_locked and local_version != s3_version

    @property
    def status(self) -> Tuple[str, str, str]:

        if self.version_in_sync:
            remote_version_message = "in sync"
        else:
            remote_version_message = "refresh required"

        if self.is_locked:
            locked_message = "locked"
        else:
            locked_message = "unlocked"

        if self.is_deployable:
            deployable_message = "out of sync"
        else:
            deployable_message = "in sync"

        return locked_message, deployable_message, remote_version_message

    @property
    def pretty_table_row(self) -> List[str]:
        return [self.component_name, self.component_type, *self.status, self.version_number]

    @property
    def next_version_number(self) -> int:
        return self.version_number + 1

    @property
    def version_in_sync(self) -> bool:
        remote_version = self.fetch_s3_version_number()
        return remote_version == self.version_number

    def increment_version(self) -> None:
        self.version_number += 1

    def component_files(self) -> List[Path]:
        file_list = []
        filters = [".DS_Store", ".empty", "__pycache__", ".zip"]

        for path in self.component_path.glob("**/*"):
            if path.is_file():
                filtered = False

                for part in path.parts:
                    if part in filters:
                        filtered = True

                if not filtered:
                    file_list.append(path)
        return file_list

    def get_relative_path(self, path: Path) -> str:
        return path.relative_to(self.root_dir).as_posix()

    def hash_file(self, path: Path) -> Tuple[str, str]:

        relative_path = self.get_relative_path(path)
        md5_hash = md5()

        with path.open("rb") as data:
            for chunk in iter(lambda: data.read(4096), b""):
                md5_hash.update(chunk)
            return relative_path, md5_hash.hexdigest()

    def get_version_hashes(self) -> Dict[str, str]:
        version_hashes = {}
        filters = [
            ".version",
            "config_merged.yml",
        ]

        for path in self.component_files():
            if path.name not in filters and path.suffix != ".zip":
                key, digest = self.hash_file(path)
                version_hashes[key] = digest
        return version_hashes

    def save_version_file(self, version_number: Optional[int] = None) -> None:
        version_hashes = self.get_version_hashes()

        if not version_number:
            version_hashes["version_number"] = self.version_number
        else:
            version_hashes["version_number"] = version_number
        json.dump(version_hashes, self.version_file.open(mode="w"), indent=4)

    def upload_object_to_s3(self, path: Path) -> None:
        s3_client = boto3.client("s3")
        relative_path = self.get_relative_path(path)
        print(f"Uploading -- s3://{self.bucket}/{self.s3_prefix}/{relative_path}")
        s3_client.upload_file(path.as_posix(), self.bucket, f"{self.s3_prefix}/{relative_path}")

    def sync(self) -> None:
        with Pool(cpu_count()) as pool:
            files = self.component_files()
            pool.map(self.upload_object_to_s3, files)

    def fetch_s3_version(self) -> Dict[str, str]:
        s3_client = boto3.client("s3")
        try:
            with BytesIO() as buffer:
                s3_client.download_fileobj(self.bucket, self.s3_version_path, buffer)
                buffer.seek(0)
                return json.load(buffer)
        except botocore.exceptions.ClientError as e:
            code = e.args[0].partition(":")[2].strip()
            if code in ["Not Found", "Forbidden"]:
                return {}
            raise e

    def lock(self) -> None:
        self.save_version_file()

    def append_version(self) -> None:
        self.increment_version()
        self.save_version_file()

    @staticmethod
    def compare_previous(previous: int, current: int) -> int:
        return max(previous, current)

    def parse_version_numbers(self, keys):
        for key in keys:
            yield int(re.match(self.version_pattern, key).group(1).partition("=")[2])

    def list_superglue_components(self) -> Generator[List[str], None, None]:
        s3_client = boto3.client("s3")
        continue_token = "start"
        while continue_token:

            kwargs = {"Bucket": self.bucket, "Prefix": self.s3_filter}

            if continue_token and continue_token != "start":
                kwargs["ContinuationToken"] = continue_token

            results = s3_client.list_objects_v2(**kwargs)

            try:
                yield [key["Key"] for key in results["Contents"] if key["Key"]]
            except KeyError:
                yield []

            continue_token = results.get("ContinuationToken")

    def fetch_s3_version_number(self) -> int:
        v = 0
        for keys in self.list_superglue_components():
            for version in self.parse_version_numbers(keys):
                v = self.compare_previous(v, version)
        return int(v)

    @abstractmethod
    def deploy(self, increment_version: bool) -> None:
        pass

    @abstractmethod
    def delete(self) -> None:
        pass

    @abstractmethod
    def save(self) -> None:
        pass



from argparse import ArgumentParser, Action, Namespace
from typing import Any, List
from superglue.core.components.project import SuperglueProject


def get_forbidden_chars(input_string: str) -> List[str]:
    forbidden = "!" "#$%&'()*+,./:;<=>?@[\\]^`{|}~-"
    captured = []
    for char in input_string:
        if char in forbidden:
            captured.append(char)
    return captured


def exit_if_job_exists(job_name: str) -> None:

    project = SuperglueProject()
    job = project.job.new(job_name)

    if job.job_path.exists():
        print(f"The job {job.job_name} already exists")
        exit(1)


#
#
# def check_module_exists(module_name: str) -> None:
#
#     project = SuperGlueProject()
#     module = SuperGlueModule(project.shared_path, module_name)
#
#     if not module.module_path.exists():
#         print(f"The module {module.module_name} does not exist")
#         exit()
#
#
class ValidateNameArgument(Action):
    def __call__(self, parser: ArgumentParser, namespace: Namespace, values: Any, option_string: str = None):
        forbidden_chars = get_forbidden_chars(values)

        if forbidden_chars:
            print(f"The {forbidden_chars} characters are not allowed in superglue component names")
            exit(1)

        setattr(namespace, self.dest, values)


#
#
# class ValidateModuleCommandName(Action):
#     def __call__(self, parser: ArgumentParser, namespace: Namespace, values: Any, option_string: str = None):
#
#         validate_input_string(values, f"The {values} is not allowed in job names")
#
#         # The module of course cannot already exist if we ar calling the new cmd
#         if not namespace.func.__name__ == "new":
#             check_module_exists(values)
#
#         setattr(namespace, self.dest, values)
